# experimentally verify the JL lemma
from sorcerun.sacred_utils import DummyRun

import numpy as np


def adapter(config, _run):
    n = config["n"]
    d = config["d"]
    k = config["k"]

    # generate n points in d dimensions on the unit sphere
    X = np.random.randn(d, n)
    norms = np.linalg.norm(X, axis=0)
    X /= norms

    # generate a Gaussian sketch matrix of size k x d
    S = np.random.randn(k, d) / np.sqrt(k)

    # sketch the data
    Y = S @ X

    # measure distortion for each data point
    new_norms = np.linalg.norm(Y, axis=0)

    distortions = np.abs(new_norms - 1)

    _run.log_scalar("mean_distortion", distortions.mean())
    _run.log_scalar("max_distortion", distortions.max())

    output = (
        f"mean distortion: {distortions.mean()}, max distortion: {distortions.max()}"
    )
    return output


adapter.experiment_name = "jl_lemma_test"

if __name__ == "__main__":
    from config import config

    print(adapter(config, DummyRun()))



from sacred import Experiment, SETTINGS
import pymongo
import traceback
from sacred.observers import MongoObserver, FileStorageObserver
from sacred.utils import apply_backspaces_and_linefeeds
import importlib
import json
from .globals import AUTH_FILE, RUNS_DIR
import sys
import os

SETTINGS.CAPTURE_MODE = "sys"

def load_python_module(python_file):
    file_dir = os.path.dirname(os.path.abspath(python_file))
    file_name = os.path.basename(python_file).replace(".py", "")

    sys.path.insert(0, file_dir)
    module = importlib.import_module(file_name)
    sys.path.pop(0)

    return module


def run_sacred_experiment(
    adapter_func,
    config,
    auth_path=AUTH_FILE,
    use_mongo=True,
    file_storage_root=".",
):
    experiment_name = getattr(adapter_func, "experiment_name", "sorcerun_experiment")
    ex = Experiment(experiment_name)
    ex.captured_out_filter = apply_backspaces_and_linefeeds

    # Read auth_data from auth_path
    if use_mongo:
        if os.path.exists(auth_path):
            with open(auth_path, "r") as f:
                auth_data = json.load(f)

            atlas = auth_data.get("atlas_connection_string", 0)
            url = (
                atlas
                if atlas != 0
                else f"mongodb://{auth_data['client_kwargs']['username']}:{auth_data['client_kwargs']['password']}@{auth_data['client_kwargs']['host']}:{auth_data['client_kwargs']['port']}",
            )
            try:
                client = pymongo.MongoClient(url)
                client.server_info()
                observer = MongoObserver(
                    # url=url,
                    client=client,
                    db_name=auth_data["db_name"],
                )

                ex.observers.append(observer)
            except Exception as e:
                traceback.print_exc()
                print(
                    f"WARNING: Failed to connect to MongoDB at {url} with above exception. Not adding mongo observer"
                )
        else:
            print(
                f"WARNING: auth file at {auth_path} does not exist. Not adding mongo observer"
            )
    else:
        print("WARNING: Not using mongo observer (use_mongo=False was passed)")

    runs_dir = os.path.join(file_storage_root, RUNS_DIR)
    os.makedirs(runs_dir, exist_ok=True)
    ex.observers.append(FileStorageObserver.create(runs_dir))
    ex.add_config(config)

    @ex.main
    def run_experiment(_config, _run):
        _run.info["info"] = "info-entry"
        result = adapter_func(_config, _run)
        return result

    ex.run()


class DummyRun():
    def __init__(self):
        pass

    def to_dict(self):
        pass

    def log_scalar(self, key, value, step=0):
        pass

    def add_artifact(self, filename, name):
        pass




import numpy as np
import itertools
from sorcerun.git_utils import get_repo, get_time_str, get_commit_hash, is_dirty

repo = get_repo()
dirty = is_dirty(repo)
time_str = get_time_str()
short_hash = get_commit_hash(repo)[:8]

extra = "jlTest"
grid_id = f"{time_str}_{extra}_{short_hash}_dirty={dirty}"


ns = (np.logspace(1, 3, 20, dtype=int)).tolist()
ds = (np.logspace(1, 3, 3, dtype=int)).tolist()
ks = (np.logspace(1, 3, 20, dtype=int)).tolist()
num_repeats = 10

grid_config = {
    "n": ns,
    "d": ds,
    "k": ks,
    "repeat": list(range(num_repeats)),
    "grid_id": [grid_id],
}


configs = []

for v in itertools.product(*grid_config.values()):
    config = dict(zip(grid_config, v))

    config["logn_over_k"] = float(np.log(config["n"])) / float(config["k"])

    configs.append(config)



import subprocess
from collections import OrderedDict
from prettytable import PrettyTable
import time
from tqdm import tqdm


class Job:
    def __init__(
        self,
        job_id,
    ):
        self.job_id = job_id
        self.job_state = None
        self.duration = None

    def __str__(self):
        return f"Job {self.job_id} is {self.job_state} for {self.duration}"

    def __repr__(self):
        return str(self)

    def update(self):
        # Get the job state
        cmd = f"sacct -j {self.job_id} -o state,elapsed --noheader"
        print(cmd)
        out = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
        self.job_state, self.duration = out.split()


def update_jobs(jobs):
    cmd = f"sacct -j {','.join([str(j.job_id) for j in jobs])} -o state,elapsed --noheader"
    out = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
    for job, line in zip(jobs, out.split("\n")):
        job.job_state, job.duration = line.split()


def aggregate_states(jobs):
    states = OrderedDict()
    states["PENDING"] = 0
    states["RUNNING"] = 0
    states["COMPLETED"] = 0

    for job in jobs:
        if job.job_state not in states:
            states[job.job_state] = 0
        states[job.job_state] += 1
    return states


def poll_jobs(jobs, poll_interval=10):
    total = len(jobs)
    print(f"Polling {total} jobs every {poll_interval} seconds\n")

    bar_format = "{desc} -- {percentage:3.0f}%|{bar}| {n_fmt}/{total_fmt}"
    update_jobs(jobs)
    time_str = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    launched = tqdm(
        total=total,
        desc=f"Launched  @ {time_str}",
        position=0,
        bar_format=bar_format,
    )
    completed = tqdm(
        total=total,
        desc=f"Completed @ {time_str}",
        position=1,
        bar_format=bar_format,
    )
    states = aggregate_states(jobs)
    first_iter = True
    while states.get("PENDING", 0) + states.get("RUNNING", 0) > 0 or first_iter:
        first_iter = False
        t = PrettyTable(["Job State", "Count"])
        time_str = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        for state, count in states.items():
            t.add_row([state, count])
            launched.n = states.get("RUNNING", 0) + states.get("COMPLETED", 0)
            launched.desc = f"Launched  @ {time_str}"
            launched.refresh()
            completed.n = states.get("COMPLETED", 0)
            completed.desc = f"Completed @ {time_str}"
            completed.refresh()
        t.align = "l"
        # print("-" * 40)
        # print(f"Job states at {time_str}")
        # print(t)
        # print("-" * 40)

        time.sleep(poll_interval)

        update_jobs(jobs)
        states = aggregate_states(jobs)

    launched.close()
    completed.close()

    print("All jobs have finished")



import time
from git.repo.base import Repo
from .globals import TIME_FORMAT


def get_repo():
    repo = Repo(".", search_parent_directories=True)
    return repo


def get_commit_hash(repo):
    commit_hash = repo.git.rev_parse("HEAD")
    return commit_hash


def is_dirty(repo):
    dirty = repo.is_dirty()
    return dirty


def get_time_str():
    time_str = time.strftime(TIME_FORMAT, time.localtime())
    return time_str


def get_exp_info(short_length=8):
    repo = get_repo()
    short_hash = get_commit_hash(repo)[:short_length]
    time_str = get_time_str()
    exp_info = f"{time_str}-{short_hash}-dirty={is_dirty(repo)}"
    return exp_info



import numpy as np
from sorcerun.git_utils import get_exp_info

exp_info = get_exp_info()

n = 1000
d = 100
k = 10
logn_over_k = float(np.log(n) / k)
config = {
    "n": n,
    "d": d,
    "k": k,
    "logn_over_k": logn_over_k,
    "exp_info": exp_info,
}
